
const EventEmitter = require("events")
const { spawn } = require("child_process")

const Logger = require("../lib/logger")
const NodeCoreUtils = require("../lib/utils")

const RTSP_TRANSPORT = ["udp", "tcp", "udp_multicast", "http"]

class NodeRelaySession extends EventEmitter {
  constructor(conf) {
    super()
    this.conf = conf
    this.id = NodeCoreUtils.generateNewSessionID()
    this.TAG = "relay"
  }

  run() {
    let format = this.conf.ouPath.startsWith("rtsp://") ? "rtsp" : "flv"
    let argv = ["-re", "-i", this.conf.inPath, "-c", "copy", "-f", format, this.conf.ouPath]

    if (this.conf.inPath[0] === "/" || this.conf.inPath[1] === ":") {
      argv.unshift("-1")
      argv.unshift("-stream_loop")
    }

    if (this.conf.inPath.startsWith("rtsp://") && this.conf.rtsp_transport) {
      if (RTSP_TRANSPORT.indexOf(this.conf.rtsp_transport) > -1) {
        argv.unshift(this.conf.rtsp_transport)
        argv.unshift("-rtsp_transport")
      }
    }

    Logger.log("[relay task] id=" + this.id, "cmd=ffmpeg", argv.join(" "))

    this.ffmpeg_exec = spawn(this.conf.ffmpeg, argv)
    this.ffmpeg_exec.on("error", (e) => {
      Logger.ffdebug(e)
    })

    this.ffmpeg_exec.stdout.on("data", (data) => {
      Logger.ffdebug(`FF输出：${data}`)
    })

    this.ffmpeg_exec.stderr.on("data", (data) => {
      Logger.ffdebug(`FF输出：${data}`)
    })

    this.ffmpeg_exec.on("close", (code) => {
      Logger.log("[relay end] id=" + this.id, "code=" + code)
      this.emit("end", this.id)
    })
  }

  end() {
    this.ffmpeg_exec.kill()
  }
}

module.exports = NodeRelaySession
